import java.awt.*;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

public class BusesGenericCollection<T extends DrawingBus, U extends IMoveableObject> {
    private int _pictureWidth;
    private int _pictureHeight;
    private final int _placeSizeWidth = 210;
    private final int _placeSizeHeight = 135;
    private SetGeneric<T> _collection;
    public static char _separatorRecords = ';';
    public static char _separatorForObject = ':';

    public BusesGenericCollection(int picWidth, int picHeight) {
        int width = picWidth / _placeSizeWidth;
        int height = picHeight / _placeSizeHeight;
        _pictureWidth = picWidth;
        _pictureHeight = picHeight;
        _collection = new SetGeneric<T>(width * height);
    }

    public void showBuses(Graphics2D g) {
        drawBackground(g);
        drawObjects(g);
    }
    public int getCount(){return _collection.getCount();}
    public boolean insert(T obj) {
        if (obj != null)
            return _collection.insert(obj);
        return false;
    }

    public boolean remove(int pos) {
        return _collection.remove(pos);
    }

    public U GetU(int pos) {
        if (_collection.Get(pos) == null)
            return null;
        return (U) _collection.Get(pos).GetMoveableObject();
    }

    public T Get(int position) {
        if (position < 0 || position >= _collection.getCount())
            return null;
        return _collection.Get(position);
    }

    private void drawBackground(Graphics2D g) {
        BasicStroke pen = new BasicStroke(3);
        g.setStroke(pen);
        for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++) {
            for (int j = 0; j < _pictureHeight / _placeSizeHeight +
                    1; ++j) {g.drawLine(i * _placeSizeWidth, j *
                        _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2, j *
                        _placeSizeHeight);
            }
            g.drawLine(i * _placeSizeWidth, 0, i *
                    _placeSizeWidth, _pictureHeight / _placeSizeHeight * _placeSizeHeight);
        }
    }

    private void drawObjects(Graphics2D g) {
        for (int i = 0; i < _collection.getCount(); i++) {
            DrawingBus bus = _collection.Get(i);
            if (bus != null) {
                int inRow = _pictureWidth / _placeSizeWidth;
                bus.setPosition(_placeSizeWidth * (inRow - 1) - (i % inRow * _placeSizeWidth), i / inRow * _placeSizeHeight);
                bus.drawTransport(g);
            }
        }
    }
    public ArrayList<T> GetBuses(){
        return _collection.GetBuses(_collection.getCount());
    }
    public boolean SaveData(File f, String name) throws IOException {
        if(f.exists()) {
            f.delete();
        }
        f.createNewFile();
        StringBuilder data = new StringBuilder();
        data.append("BusCollection\n");
        data.append(String.format("%s\n", name));
        StringBuilder records = new StringBuilder();
        for(DrawingBus elem : GetBuses())
        {
            records.append(String.format("%s%c", ExtentionDrawingBus.GetDataForSave(elem, _separatorForObject),
                    _separatorRecords));
        }
        data.append(records);
        if(data.length() == 0)
            return false;
        FileWriter writer = new FileWriter(f);
        writer.write(data.toString());
        writer.flush();
        writer.close();
        return true;
    }
    public void Clear() {
        _collection = new SetGeneric<>(_pictureWidth * _pictureHeight);

    }
}

