import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;
import javax.xml.crypto.dsig.keyinfo.KeyValue;
import java.io.*;
import java.util.*;
import java.util.Queue;
public class BusesGenericStorage {
    public HashMap<String, BusesGenericCollection<DrawingBus, DrawingObjectBus>> _busStorages;
    private static final char _separatorForKeyValue = '|';
    private final char _separatorRecords = ';';
    private static final char _separatorForObject = ':';
    public boolean SaveData(File f) throws IOException {
        if(f.exists()) {
            f.delete();
        }
        f.createNewFile();
        StringBuilder data = new StringBuilder();
        data.append("BusStorage\n");
        for(Map.Entry<String, BusesGenericCollection<DrawingBus, DrawingObjectBus>> record : _busStorages.entrySet()){
            StringBuilder records = new StringBuilder();
            for(DrawingBus elem : record.getValue().GetBuses())
            {
                records.append(String.format("%s%c", ExtentionDrawingBus.GetDataForSave(elem, _separatorForObject),
                        _separatorRecords));
            }
            data.append(String.format("%s%c%s\n", record.getKey(), _separatorForKeyValue, records.toString()));
        }
        if(data.length() == 0)
            return false;
        FileWriter writer = new FileWriter(f);
        writer.write(data.toString());
        writer.flush();
        writer.close();
        return true;
    }
    public boolean LoadData(File f) throws FileNotFoundException {
        if(!f.exists())
            return false;
        StringBuilder bufferTextFromFile =new StringBuilder();
        Scanner s = new Scanner(f);
        while(s.hasNext())
            bufferTextFromFile.append(s.next() + "\n");
        s.close();
        var strs = bufferTextFromFile.toString().split("\n");
        if(strs == null || strs.length == 0)
            return false;
        if (!strs[0].startsWith("BusStorage"))
            return false;
        _busStorages.clear();
        for(String data : strs){
            String st = new String("\\" + Character.toString( _separatorForKeyValue));
            String[]record = data.split(st);
            if (record.length != 2)
                continue;
            BusesGenericCollection<DrawingBus, DrawingObjectBus> collection =
                    new BusesGenericCollection<>(_pictureWidth, _pictureHeight);
            String[] set = record[1].split(Character.toString(_separatorRecords));

            for(int i = set.length -1; i >=0; i--){
                String elem = set[i];
                DrawingBus bus = ExtentionDrawingBus.CreateDrawingBus(elem,
                        _separatorForObject, _pictureWidth, _pictureHeight);
                if (bus != null)
                {
                    if (!(collection.insert(bus)))
                    {
                        return false;
                    }
                }
            }
            _busStorages.put(record[0], collection);
        }
        return true;
    }

    public boolean LoadCollection(File f) throws FileNotFoundException {
        if(!f.exists())
            return false;
        StringBuilder bufferTextFromFile =new StringBuilder();
        Scanner s = new Scanner(f);
        while(s.hasNext())
            bufferTextFromFile.append(s.next() + "\n");
        s.close();
        var strs = bufferTextFromFile.toString().split("\n");
        if(strs == null || strs.length == 0)
            return false;
        if (!strs[0].startsWith("BusCollection"))
            return false;
        String collectionName = strs[1];
        BusesGenericCollection<DrawingBus, DrawingObjectBus> collection = getCollection(collectionName);
        if(collection == null)
            collection = new BusesGenericCollection<>(_pictureWidth, _pictureHeight);
        else
            collection.Clear();
        String[] busesInfo = strs[2].split(Character.toString(BusesGenericCollection._separatorRecords));
        for(int i = busesInfo.length-1; i >= 0; i--){
            String data = busesInfo[i];
            DrawingBus bus = ExtentionDrawingBus.CreateDrawingBus(data,
                    BusesGenericCollection._separatorForObject, _pictureWidth, _pictureHeight);
            if (bus != null)
            {
                if (!(collection.insert(bus)))
                {
                    return false;
                }
            }
        }
        AddSetFromFile(collectionName, collection);
        return true;
    }
    public List<String> Keys(){
        if(_busStorages == null)
            return null;
        return _busStorages.keySet().stream().collect(Collectors.toList());
    }
    private final int _pictureWidth;
    private final int _pictureHeight;

    public BusesGenericStorage(int pictureWidth, int pictureHeight){
        _busStorages = new HashMap<>();
        _pictureWidth = pictureWidth;
        _pictureHeight = pictureHeight;
    }

    public void addSet(String name){
        if(_busStorages.containsKey(name))
            return;
        _busStorages.put(name, new BusesGenericCollection<>(_pictureWidth, _pictureHeight));
    }

    public void delSet(String name, Queue<DrawingBus> trashBox){
        if(!_busStorages.containsKey(name))
            return;
        BusesGenericCollection<DrawingBus, DrawingObjectBus> cur = _busStorages.get(name);
        for(int i = 0; i < cur.getCount(); i++)
            trashBox.add(cur.Get(i));
        _busStorages.remove(name);
    }

    public BusesGenericCollection<DrawingBus, DrawingObjectBus> getCollection(String name){
        if(!_busStorages.containsKey(name))
            return null;
        return _busStorages.get(name);
    }

    public DrawingBus Get(String collectionName, int position){
        return _busStorages.get(collectionName).Get(position);
    }
    public void AddSetFromFile(String name, BusesGenericCollection<DrawingBus, DrawingObjectBus> toAdd){
        if(_busStorages.containsKey(name)){
            _busStorages.remove(name);
        }
        _busStorages.put(name, toAdd);
    }
    public BusesGenericCollection<DrawingBus, DrawingObjectBus> GetCollection(String collectionName){
        return _busStorages.get(collectionName);
    }
}
