import javax.swing.*;
import javax.swing.border.StrokeBorder;
import java.awt.*;
import java.io.IOException;
import java.util.List;
import java.util.Objects;
import java.util.ArrayDeque;
import java.util.Queue;
import java.io.File;

public class FrameBusCollection extends JFrame {
    private BusesGenericStorage storage;
    private JList<String> listBoxStorages;
    private DefaultListModel<String> listBoxModel;
    JComponent pictureBoxCollection;
    TextField textFieldNumber;
    TextField textFieldStorageName;
    Queue<DrawingBus> trashCollection = new ArrayDeque<>();

    public FrameBusCollection() {
        super("Набор автобусов");
        setSize(new Dimension(1000, 500));
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        createGui();
        storage = new BusesGenericStorage(pictureBoxCollection.getWidth(), pictureBoxCollection.getHeight());
        setVisible(true);
    }

    private void createGui() {
        pictureBoxCollection = new JComponent() {
            public void paintComponent(Graphics graphics) {
                super.paintComponent(graphics);
                Graphics2D graphics2D = (Graphics2D) graphics;
                if (listBoxStorages == null || storage == null)
                    return;
                var collection = storage.getCollection(listBoxStorages.getSelectedValue());
                if (collection == null)
                    return;
                collection.showBuses(graphics2D);
                super.repaint();
            }
        };
        JMenuBar menuFile = new JMenuBar();
        JMenu file = new JMenu("Файл");
        menuFile.add(file);
        JMenuItem saveFile = new JMenuItem("Сохранить");
        JMenuItem loadFile = new JMenuItem("Загрузить");
        JMenuItem saveCollection = new JMenuItem("Сохранить коллекцию");
        JMenuItem loadCollection = new JMenuItem("Загрузить коллекцию");
        file.add(saveCollection);
        file.add(loadCollection);
        file.add(saveFile);
        file.add(loadFile);
        setJMenuBar(menuFile);
        pictureBoxCollection.setSize(new Dimension(700, 450));
        JButton buttonAddBus = new JButton("Добавить автобус");
        textFieldNumber = new TextField();
        textFieldStorageName = new TextField();
        JScrollPane scrollPane = new JScrollPane();
        JButton buttonRemoveBus = new JButton("Удалить автобус");
        JButton buttonRefreshCollection = new JButton("Обновить коллекцию");
        JButton buttonAddCollection = new JButton("Добавить набор");
        listBoxModel = new DefaultListModel<>();
        listBoxStorages = new JList<>(listBoxModel);
        scrollPane.setViewportView(listBoxStorages);
        JButton buttonDeleteCollection = new JButton("Удалить набор");
        JButton buttonTrash = new JButton("Корзина");
        buttonAddBus.addActionListener(e -> buttonAddBusClick());
        buttonRemoveBus.addActionListener(e -> buttonRemoveBusClick());
        buttonRefreshCollection.addActionListener(e -> buttonRefreshBusClick());
        buttonAddCollection.addActionListener(e -> buttonAddCollectionClick());
        buttonDeleteCollection.addActionListener(e -> buttonDeleteCollectionClick());
        buttonTrash.addActionListener(e -> buttonTrashClick());
        saveFile.addActionListener(e -> saveFile());
        saveCollection.addActionListener(e -> saveCollection());
        loadFile.addActionListener(e -> loadFile());
        loadCollection.addActionListener(e -> loadCollection());
        JPanel panelTools = new JPanel(new GridBagLayout());
        panelTools.setBorder(new StrokeBorder(new BasicStroke(3)));
        JPanel panelCollection = new JPanel(new GridBagLayout());
        panelCollection.setBorder(new StrokeBorder(new BasicStroke(3)));
        GridBagConstraints constraints = new GridBagConstraints();
        constraints.insets.left = constraints.insets.right = 5;
        constraints.insets.top = constraints.insets.bottom = 4;
        constraints.fill = GridBagConstraints.BOTH;
        constraints.gridx = 0;
        constraints.gridy = 0;
        panelCollection.add(textFieldStorageName, constraints);
        constraints.gridx = 0;
        constraints.gridy = 1;
        panelCollection.add(buttonAddCollection, constraints);
        constraints.gridx = 0;
        constraints.gridy = 2;
        panelCollection.add(scrollPane, constraints);
        constraints.gridx = 0;
        constraints.gridy = 3;
        panelCollection.add(buttonDeleteCollection, constraints);
        constraints.gridx = 0;
        constraints.gridy = 0;
        panelTools.add(panelCollection, constraints);
        constraints.gridx = 0;
        constraints.gridy = 1;
        panelTools.add(buttonAddBus, constraints);
        constraints.gridx = 0;
        constraints.gridy = 2;
        panelTools.add(textFieldNumber, constraints);
        constraints.gridx = 0;
        constraints.gridy = 3;
        panelTools.add(buttonRemoveBus, constraints);
        constraints.gridx = 0;
        constraints.gridy = 4;
        panelTools.add(buttonRefreshCollection, constraints);
        constraints.gridx = 0;
        constraints.gridy = 5;
        panelTools.add(buttonTrash, constraints);
        setLayout(new BorderLayout());
        add(panelTools, BorderLayout.EAST);
        add(pictureBoxCollection, BorderLayout.CENTER);
    }

    private void buttonAddBusClick() {
        if (listBoxStorages.getSelectedIndex() == -1) {
            return;
        }
        BusesGenericCollection<DrawingBus, DrawingObjectBus> drawingBuses = storage.getCollection(listBoxStorages.getSelectedValue());
        FrameBusConfig frameBusConfig = new FrameBusConfig();
        frameBusConfig.addButton.addActionListener(e -> {
            if (drawingBuses.insert(frameBusConfig.drawingBus)) {
                frameBusConfig.dispose();
                frameBusConfig.drawingBus.pictureWidth = pictureBoxCollection.getWidth();
                frameBusConfig.drawingBus.pictureHeight = pictureBoxCollection.getHeight();
                JOptionPane.showMessageDialog(null, "Объект добавлен", "Информация", JOptionPane.INFORMATION_MESSAGE);
                pictureBoxCollection.repaint();
            } else {
                JOptionPane.showMessageDialog(null, "Не удалось добавить объект", "Информация", JOptionPane.INFORMATION_MESSAGE);
            }
        });
        frameBusConfig.cancelButton.addActionListener(e -> frameBusConfig.dispose());
    }

    private void buttonRemoveBusClick() {
        if (listBoxStorages.getSelectedIndex() == -1)
            return;
        var obj = storage.getCollection(listBoxStorages.getSelectedValue());
        if (obj == null)
            return;
        int pos = Integer.parseInt(textFieldNumber.getText());
        trashCollection.add(obj.Get(pos));
        if (obj.remove(pos)) {
            JOptionPane.showMessageDialog(this, "Объект удалён");
            pictureBoxCollection.repaint();
        } else {
            JOptionPane.showMessageDialog(this, "Не удалось удалить объект");
        }
    }

    private void buttonRefreshBusClick() {
        pictureBoxCollection.repaint();
    }

    private void reloadObjects() {
        int index = listBoxStorages.getSelectedIndex();
        listBoxModel.clear();
        List<String> keys = storage.Keys();
        for (String key : keys) {
            listBoxModel.addElement(key);
        }
        if (listBoxModel.size() > 0 && (index == -1 || index >= listBoxModel.size()))
            listBoxStorages.setSelectedIndex(0);
        else if (listBoxModel.size() > 0)
            listBoxStorages.setSelectedIndex(index);
    }

    private void buttonAddCollectionClick() {
        if (textFieldStorageName.getText() == null) {
            JOptionPane.showMessageDialog(this, "Не все данные заполнены");
            return;
        }
        String name = textFieldStorageName.getText();
        if (Objects.equals(name, "")) {
            JOptionPane.showMessageDialog(this, "Не все данные заполнены");
            return;
        }
        storage.addSet(name);
        reloadObjects();
    }

    private void buttonDeleteCollectionClick() {
        if (listBoxStorages.getSelectedIndex() == -1)
            return;
        storage.delSet(listBoxStorages.getSelectedValue(), trashCollection);
        reloadObjects();
    }

    private void buttonTrashClick() {
        if (trashCollection.size() == 0)
            return;
        FrameTrolleybus frame;
        try {
            frame = new FrameTrolleybus();
            frame.ChangeTrolleybus(trashCollection.peek());
            trashCollection.remove();
        } catch (IOException e) {
            throw new RuntimeException();
        }
    }

    private void saveFile() {
        JFileChooser fc = new JFileChooser("C:\\Users\\niyaz\\OneDrive\\Документы");
        fc.addChoosableFileFilter(new TxtSaveFilter());
        int retrieval = fc.showSaveDialog(null);

        if (retrieval == JFileChooser.APPROVE_OPTION) {
            File file = new File(fc.getSelectedFile() + "." + "txt");

            try {
                storage.SaveData(file);
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }
    }

    private void saveCollection() {
        JFileChooser fc = new JFileChooser("C:\\Users\\niyaz\\OneDrive\\Документы");
        fc.addChoosableFileFilter(new TxtSaveFilter());
        int retrieval = fc.showSaveDialog(null);
        if (retrieval == JFileChooser.APPROVE_OPTION) {
            File file = new File(fc.getSelectedFile() + "." + "txt");

            try {
                if (listBoxStorages.getSelectedIndex() == -1) {
                    return;
                }
                storage._busStorages.get(listBoxStorages.getSelectedValue()).SaveData(file, listBoxStorages.getSelectedValue());
                reloadObjects();
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }
    }

    private void loadFile() {
        JFileChooser fc = new JFileChooser("C:\\Users\\niyaz\\OneDrive\\Документы");
        fc.addChoosableFileFilter(new TxtSaveFilter());
        int ret = fc.showDialog(null, "Открыть файл");
        if (ret == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            try {
                storage.LoadData(file);
                reloadObjects();
                pictureBoxCollection.repaint();
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }

    }

    private void loadCollection() {
        JFileChooser fc = new JFileChooser("C:\\Users\\niyaz\\OneDrive\\Документы");
        int ret = fc.showDialog(null, "Открыть файл");
        if (ret == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            try {
                storage.LoadCollection(file);
                reloadObjects();
                pictureBoxCollection.repaint();
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }

    }
}
